@@ -1,15 +1,15 @@
 #
 # ex1: simple use of Config::Validator with Getopt::Long
 #
-# $ perl ex1.pl --help
+# $ perl ex1.pl -h
 # $ perl ex1.pl --src-host foo --dst-host bar
 #
 
 use strict;
 use warnings;
-use Config::Validator;
-use Data::Dumper;
-use Getopt::Long;
+use Config::Validator qw();
+use Data::Dumper qw(Dumper);
+use Getopt::Long qw(GetOptions);
 
 our($Validator, @Options, %Config);
 
@@ -22,13 +22,12 @@ $Validator = Config::Validator->new({
 	"debug"    => { type => "integer", optional => "true" },
 	"dst-port" => { type => "integer", min => 0, max => 65535, optional => "true" },
 	"dst-host" => { type => "string", match => qr/^[\w\-\.]+$/ },
-	"help"     => { type => "boolean", optional => "true" },
 	"src-port" => { type => "integer", min => 0, max => 65535, optional => "true" },
 	"src-host" => { type => "string", match => qr/^[\w\-\.]+$/ },
     },
 });
 
-@Options = sort($Validator->options());
+@Options = sort($Validator->options(), "help|h|?");
 
 GetOptions(\%Config, @Options) or die;
 
